// fixup of WRLD CELL REGN after moving to TES5Edit

unit UserScript;

const
  plugin_name = 'morr.esp';

var
  dumppath: string;
  sl, slv: TStringList;
  plugin: IInterface;

// helper function, finds Region Data Entry of provided type in REGN Region Data Entries
function DataEntryFromREGN(r: IInterface; typ: string): IInterface;
var
  entries, e: IInterface;
  i: integer;
begin
  Result := nil;
  entries := ElementByName(r, 'Region Data Entries');
  for i := 0 to ElementCount(entries) - 1 do begin
    e := ElementByIndex(entries, i);
    if GetElementEditValues(e, 'RDAT\Type') = typ then begin
      Result := e;
      Exit;
    end;
  end;
end;

//============================================================================
procedure AfterREGN;
var
  i: integer;
  regns, rdata, r, ent: IInterface;
  baserecord: IInterface;
begin
  regns := GroupBySignature(plugin, 'REGN');
  if not Assigned(regns) then begin
    AddMessage('No REGN group found');
    Exit;
  end;

  // WeatherFallForest [REGN:000C5857]
  baserecord := RecordByFormID(FileByIndex(0), $000C5857, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find REGN base record');
    Exit;
  end;
  
  for i := 0 to ElementCount(regns) - 1 do begin
    r := ElementByIndex(regns, i);
    // change weather/sound only in Morrowind worldspace
    if GetElementNativeValues(r, 'WNAM') <> $01380000 then Continue;
    rdata := ElementByName(r, 'Region Data Entries');

    if not Assigned(DataEntryFromREGN(r, 'Weather')) then begin
      ent := ElementAssign(rdata, HighInteger, nil, False);
      ElementAssign(ent, LowInteger, DataEntryFromREGN(baserecord, 'Weather'), False);
    end;

    if not Assigned(DataEntryFromREGN(r, 'Sound')) then begin
      ent := ElementAssign(rdata, HighInteger, nil, False);
      ElementAssign(ent, LowInteger, DataEntryFromREGN(baserecord, 'Sound'), False);
    end;
  end;
end;  

//============================================================================
procedure AfterWRLD;
var
  wrlds, r: IInterface;
  baserecord: IInterface;
  formid: Cardinal;
  i: integer;
  flags: integer;
begin
  wrlds := GroupBySignature(plugin, 'WRLD');
  if not Assigned(wrlds) then begin
    AddMessage('No WRLD group found');
    Exit;
  end;
  
  for i := 0 to ElementCount(wrlds) - 1 do begin
    r := ElementByIndex(wrlds, i);
    
    // common values
    SetElementEditValues(r, 'ONAM\World Map Scale', '1');
    // distand lod multiplier
    Add(r, 'NAMA', True);
    SetElementNativeValues(r, 'NAMA', '1');

    flags := GetElementNativeValues(r, 'DATA');
    // small world flag?
    if flags and $01 = 0 then
      // Tamriel "Skyrim" [WRLD:0000003C]
      formid := $0000003C
    else
      // WindhelmWorld "Windhelm" [WRLD:0001691D]
      formid := $0001691D;
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex64(formid, 8));
      Exit;
    end;
    
    // water
    if ElementExists(baserecord, 'NAM2') then begin
      Add(r, 'NAM2', True);
      SetElementNativeValues(r, 'NAM2', GetElementNativeValues(baserecord, 'NAM2'));
    end;
    // lod water
    if ElementExists(baserecord, 'NAM3') then begin
      Add(r, 'NAM3', True);
      SetElementNativeValues(r, 'NAM3', GetElementNativeValues(baserecord, 'NAM3'));
    end;
    // lod water height
    if ElementExists(baserecord, 'NAM4') then begin
      Add(r, 'NAM4', True);
      SetElementNativeValues(r, 'NAM4', GetElementNativeValues(baserecord, 'NAM4'));
    end;
    // lighting template
    if ElementExists(baserecord, 'LTMP') then begin
      Add(r, 'LTMP', True);
      SetElementNativeValues(r, 'LTMP', GetElementNativeValues(baserecord, 'LTMP'));
    end;
    // climate
    if ElementExists(baserecord, 'CLMT') then begin
      Add(r, 'CLMT', True);
      SetElementNativeValues(r, 'CLMT', GetElementNativeValues(baserecord, 'CLMT'));
    end;
    // music
    if ElementExists(baserecord, 'ZNAM') then begin
      Add(r, 'ZNAM', True);
      SetElementNativeValues(r, 'ZNAM', GetElementNativeValues(baserecord, 'ZNAM'));
    end;
    // another set of flags
    if ElementExists(baserecord, 'PNAM') then begin
      Add(r, 'PNAM', True);
      SetElementNativeValues(r, 'PNAM', GetElementNativeValues(baserecord, 'PNAM'));
    end;
    // land data
    if ElementExists(baserecord, 'DNAM') then begin
      Add(r, 'DNAM', True);
      SetElementNativeValues(r, 'DNAM\Default Land Height', GetElementNativeValues(baserecord, 'DNAM\Default Land Height'));
      SetElementNativeValues(r, 'DNAM\Default Water Height', GetElementNativeValues(baserecord, 'DNAM\Default Water Height'));
    end;

  end;
  

end;  

//============================================================================
procedure AfterCELL;
var
  i, flags: integer;
  IsInterior: boolean;
  baserecord, r, sr: IInterface;
  formid: Cardinal;
  s1: string;
begin
  sl.LoadFromFile(dumppath + 'Skywind CELL.txt');
  for i := 0 to sl.Count - 1 do begin
    //if i = 100 then Exit;
    slv.DelimitedText := sl[i];
    s1 := slv[0];

    // new cell entry
    if s1[1] = '0' then begin
      r := RecordByFormID(plugin, StrToInt64('$' + s1), True);
      if not Assigned(r) then begin
        AddMessage('Can not find CELL record ' + s1);
        Exit;
      end;
      // set cell flags
      flags := StrToInt64('$' + slv[1]);
      flags := flags and $FFBF; // clear "hand changed" flag, looks like Skyrim doesn't use it
      SetElementNativeValues(r, 'DATA', flags);
      if flags and 1 > 0 then IsInterior := True else IsInterior := False;
      // set water level
      if slv[3] <> '' then begin
        Add(r, 'XCLW', True);
        SetElementEditValues(r, 'XCLW', slv[3]);
      end;

      // cell type template decider
      if not IsInterior then
        // [CELL:00000D74] (in Tamriel "Skyrim" [WRLD:0000003C] at 0,0)
        formid := $00000D74
      else
      if slv[2] = 'Dungeon' then
        // GeirmundsHall01 "Geirmund's Hall" [CELL:000A5A71]
        formid := $000A5A71
      else
      if slv[2] = 'Public' then
        // RiftenTempleofMara "Temple of Mara" [CELL:00016BD7]
        formid := $00016BD7
      else
        // IvarsteadFellstarFarm "Fellstar Farm" [CELL:00013A48]
        formid := $00013A48;

      baserecord := RecordByFormID(FileByIndex(0), formid, True);
      if not Assigned(baserecord) then begin
        AddMessage('Can not find base record ' + IntToHex64(formid, 8));
        Exit;
      end;

      // lighting
      if ElementExists(baserecord, 'XCLL') then begin
        sr := Add(r, 'XCLL', True);
        ElementAssign(sr, LowInteger, ElementBySignature(baserecord, 'XCLL'), False);
        SetElementNativeValues(r, 'XCLL\Inherits', 0); // clear inheritance flags
      end;
      // lighting template
      if ElementExists(baserecord, 'LTMP') then begin
        Add(r, 'LTMP', True);
        SetElementNativeValues(r, 'LTMP', GetElementNativeValues(baserecord, 'LTMP'));
      end;
      // acoustic space
      if ElementExists(baserecord, 'XCAS') then begin
        Add(r, 'XCAS', True);
        SetElementNativeValues(r, 'XCAS', GetElementNativeValues(baserecord, 'XCAS'));
      end;
      // music type
      if ElementExists(baserecord, 'XCMO') then begin
        Add(r, 'XCMO', True);
        SetElementNativeValues(r, 'XCMO', GetElementNativeValues(baserecord, 'XCMO'));
      end;
      // image space
      if ElementExists(baserecord, 'XCIM') then begin
        Add(r, 'XCIM', True);
        SetElementNativeValues(r, 'XCIM', GetElementNativeValues(baserecord, 'XCIM'));
      end;
      
    end else
    // new subrecord
    if slv.Count = 1 then begin
      if Assigned(r) then
        Add(r, s1, True);
    end else
    // value
    if slv.Count = 2 then begin
      if Assigned(r) then
        SetElementEditValues(r, s1, slv[1]);
    end;
  end;
end;  

//============================================================================
procedure AfterREFR;
var
  i: integer;
  r: IInterface;
  s1: string;
begin
  sl.LoadFromFile(dumppath + 'Skywind REFR.txt');
  for i := 0 to sl.Count - 1 do begin
    //if i = 100 then Exit;
    slv.DelimitedText := sl[i];
    s1 := slv[0];

    // new refr entry
    if s1[1] = '0' then begin
      r := RecordByFormID(plugin, StrToInt64('$' + s1), True);
      if not Assigned(r) then begin
        AddMessage('Can not find REFR record ' + s1);
        Exit;
      end;
    end else
    // new subrecord
    if slv.Count = 1 then begin
      if Assigned(r) then
        Add(r, s1, True);
    end else
    // value
    if slv.Count = 2 then begin
      if Assigned(r) then begin
        if (s1 = 'XTEL\Door') or (s1 = 'XLOC\Key') then
          SetElementNativeValues(r, s1, StrToInt64('$' + slv[1]))
        else
        if (s1 = 'XLOC\Level') or (s1 = 'XLOC\Flags') then
          SetElementNativeValues(r, s1, StrToInt(slv[1]))
        else
          SetElementEditValues(r, s1, slv[1]);
      end;
    end;
    
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;
  
  dumppath := ProgramPath + 'Edit Scripts\';
  
  plugin := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(plugin), plugin_name) then begin
    AddMessage(plugin_name + ' must be loaded last');
    Result := 1;
    Exit;
  end;

  AfterREGN;
  AfterWRLD;
  AfterCELL;
  AfterREFR;
  
  sl.Free;
  slv.Free;
  Result := 1;
end;

end.
