// transfer CONT from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;
  slObl, slSky: TStringList;

//============================================================================
function Obl2Sky(FormID: Cardinal): Cardinal;
var
  i: integer;
begin
  if not Assigned(slObl) then begin
    //AddMessage('Initializing converter');
    slObl := TStringList.Create;
    slSky := TStringList.Create;
    sl.LoadFromFile(ProgramPath + 'Edit Scripts\OblivionSkyrim.txt');
    for i := 0 to sl.Count - 1 do begin
      slv.DelimitedText := sl[i];
      if slv.Count < 1 then Continue;
      slObl.Add(slv[0]);
      if slv.Count > 2 then
        slSky.Add(slv[2])
      else
        //GlazedCup01 "Cup" [MISC:000B9BDC]
        slSky.Add('000B9BDC');
    end;
  end;
  
  if (FormID shr 24) > 0 then begin
    Result := FormID;
    Exit;
  end;
  
  i := slObl.IndexOf(IntToHex64(FormID, 8));
  if i <> -1 then
    Result := StrToInt64('$' + slSky[i])
  else
    Result := $000B9BDC;

end;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r, lst, entry: IInterface;
  i, cnt: integer;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // BarrelFood01 "Barrel" [CONT:00000845]
  baserecord := RecordByFormID(FileByIndex(0), $00000845, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find base record');
    Result := 1;
    Exit;
  end;
  
  // init converter
  Obl2Sky(0);
  
  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    if slv[0] <> '+' then begin
      // set count for previous record
      if Assigned(lst) then begin
        cnt := ElementCount(lst);
        if cnt > 0 then
          SetElementNativeValues(r, 'COCT', cnt)
        else begin
          RemoveElement(r, 'COCT');
          RemoveElement(r, 'Items');
        end;
      end;
      // copy base record as new
      r := wbCopyElementToFile(baserecord, skywind, True, True);
      if not Assigned(r) then begin
        AddMessage('Unable to copy base record as new');
        Exit;
      end;
      //RemoveElement(r, 'COCT'); Exit;
      
      
      SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
      SetElementEditValues(r, 'EDID', slv[1]);
      SetElementEditValues(r, 'FULL', slv[2]);
      SetElementEditValues(r, 'Model\MODL', slv[3]);
      SetElementNativeValues(r, 'DATA\Flags', StrToInt(slv[4]));
      SetElementEditValues(r, 'DATA\Weight', slv[5]);
      if slv[6] <> '' then SetElementNativeValues(r, 'SNAM', StrToInt64('$' + slv[6]));
      if slv[7] <> '' then SetElementNativeValues(r, 'QNAM', StrToInt64('$' + slv[7]));
      lst := ElementByName(r, 'Items');
      // clear list
      while ElementCount(lst) > 0 do RemoveElement(lst, 0);
    end else begin
      // add new entry at the end of list
      entry := ElementAssign(lst, HighInteger, nil, False);
      SetElementNativeValues(entry, 'CNTO\Item', Obl2Sky(StrToInt64('$' + slv[1])));
      SetElementEditValues(entry, 'CNTO\Count', slv[2]);
    end;
  end;
  
  // set count for last record
  if Assigned(lst) then begin
    cnt := ElementCount(lst);
    if cnt > 0 then
      SetElementNativeValues(r, 'COCT', cnt)
    else begin
      RemoveElement(r, 'COCT');
      RemoveElement(r, 'Items');
    end;
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind CONT.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  snam, qnam: string;
  i, cnt: integer;
  lst, item: IInterface;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'CONT' then
    Exit;

  // export
  snam := IntToHex(GetElementNativeValues(e, 'SNAM'), 8);
  if Copy(snam, 1, 2) <> '01' then snam := '';
  qnam := IntToHex(GetElementNativeValues(e, 'QNAM'), 8);
  if Copy(qnam, 1, 2) <> '01' then qnam := '';
  line := Format('%s;%s;%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    IntToStr(GetElementNativeValues(e, 'DATA\Flags')),
    GetElementEditValues(e, 'DATA\Weight'),
    snam,
    qnam
  ]);
  sl.Add(line);
  
  lst := ElementByName(e, 'Items');
  if not Assigned(lst) then
    Exit;
  
  for i := 0 to ElementCount(lst) - 1 do begin
    item  := ElementByIndex(lst, i);
    // MASTERFILE: Container object (000B9BDC) has a count of less then 1 so no item is really added but will cause issues in game on owner object "0comUchestU02UvUalusaron".
    // CK doesn't like negative and zero counts
    cnt := GetElementNativeValues(item, 'Count');
    if cnt <= 0 then cnt := 1; 
    line := Format('+;%s;%s', [
      IntToHex64(GetElementNativeValues(item, 'Item'), 8),
      IntToStr(cnt)
    ]);
    sl.Add(line);
  end;
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
  if Assigned(slObl) then slObl.Free;
  if Assigned(slSky) then slSky.Free;
end;

end.
