// transfer CREA from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
  typ, formid: Cardinal;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    
    typ := StrToInt('$' + slv[3]);
    
    // daedra, EncDremoraMelee01 "Dremora Churl" [NPC_:00023A95]
    if typ = 1 then formid := $00023A95 else
    // undead, EncSkeleton01Melee1H "Skeleton" [NPC_:0002D1DE]
    if typ = 2 then formid := $0002D1DE else
    // humanoid, EncBandit00Template "Bandit" [NPC_:00039CF4]
    if typ = 3 then formid := $00039CF4 else
    // horse, EncHorseGrey "Horse" [NPC_:00068D6D]
    if typ = 4 then formid := $00068D6D else
    // creature, anything else, EncSkeever "Skeever" [NPC_:00023AB7]
      formid := $00023AB7;
    
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Result := 1;
      Exit;
    end;
   
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    SetElementEditValues(r, 'FULL', slv[2]);
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind CREA.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'CREA' then
    Exit;

  line := Format('%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    IntToHex64(GetElementNativeValues(e, 'DATA\Type'), 8)
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
