unit UserScript;

var
  plgpatch, plgcot, plgskywind: IInterface;

// helper function, finds Region Data Entry of provided type in REGN Region Data Entries
function DataEntryFromREGN(r: IInterface; typ: string): IInterface;
var
  entries, e: IInterface;
  i: integer;
begin
  Result := nil;
  entries := ElementByName(r, 'Region Data Entries');
  for i := 0 to ElementCount(entries) - 1 do begin
    e := ElementByIndex(entries, i);
    if GetElementEditValues(e, 'RDAT\Type') = typ then begin
      Result := e;
      Exit;
    end;
  end;
end;

//============================================================================
procedure PatchREGN;
var
  i: integer;
  regns, rdata, r, ent: IInterface;
  baserecord: IInterface;
  regid: string;
  formid: Cardinal;
begin
  regns := GroupBySignature(plgskywind, 'REGN');
  if not Assigned(regns) then begin
    AddMessage('No REGN group found');
    Exit;
  end;

  for i := 0 to ElementCount(regns) - 1 do begin
    r := ElementByIndex(regns, i);
    
    // copy Skywind region to patch
    AddRequiredElementMasters(r, plgpatch, False);
    r := wbCopyElementToFile(r, plgpatch, False, True);
    
    if not Assigned(r) then begin
      AddMessage('Can not copy region to patch');
      Exit;
    end;
    
    regid := GetElementEditValues(r, 'EDID');
    
    if
      SameText(regid, 'FelsaadCoastRegion') or
      SameText(regid, 'MoesringMountainsRegion') or
      SameText(regid, 'MoesringMountainsRegion') or
      SameText(regid, 'IsinfierPlainsRegion') or
      SameText(regid, 'ThirskRegion') or
      SameText(regid, 'RavenRockRegion') or
      SameText(regid, 'SkaalVillageRegion')
    then
      formid := $000C5853  // WeatherSnow [REGN:000C5853]
    else
    if SameText(regid, '0MournholdRegion') then formid := $000C5856  // WeatherReach [REGN:000C5856]
    else
      formid := $0010474D;  // WeatherFalkreath [REGN:0010474D]
      //formid := $000C5854;  // WeatherVolcanicTundra [REGN:000C5854]

    baserecord := RecordByFormID(plgcot, formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find COT REGN base record ' + IntToHex64(formid, 8));
      Exit;
    end;
    
    rdata := ElementByName(r, 'Region Data Entries');
    if not Assigned(rdata) then begin
      rdata := Add(r, 'Region Data Entries', True);
      RemoveElement(rdata, 0);
    end;

    ent := DataEntryFromREGN(r, 'Weather');
    if not Assigned(ent) then
      ent := ElementAssign(rdata, HighInteger, nil, False);
    ElementAssign(ent, LowInteger, DataEntryFromREGN(baserecord, 'Weather'), False);

    ent := DataEntryFromREGN(r, 'Sound');
    if not Assigned(ent) then
      ent := ElementAssign(rdata, HighInteger, nil, False);
    ElementAssign(ent, LowInteger, DataEntryFromREGN(baserecord, 'Sound'), False);
    
  end;
end;  

//============================================================================
function Initialize: integer;
var
  i: integer;
  s: string;
  f: IInterface;
begin
  for i := 0 to FileCount - 1 do begin
    f := FileByIndex(i);
    s := GetFileName(f);
    if SameText(s, 'ClimatesOfTamriel.esm') then plgcot := f else
    if SameText(s, 'Skywind.esm') then plgskywind := f else
    if SameText(s, 'SkywindCOT.esp') then plgpatch := f;
  end;

  if not (Assigned(plgcot) and Assigned(plgskywind) and Assigned(plgpatch)) then begin
    AddMessage('Not all required plugins loaded');
    Result := 1;
    Exit;
  end;

  PatchREGN;

  Result := 1;
end;

end.
