// transfer DOOR from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // FarmhouseDoor01 "Door" [DOOR:00012EB1]
  baserecord := RecordByFormID(FileByIndex(0), $00012EB1, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find base record');
    Result := 1;
    Exit;
  end;
  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;
    slv.DelimitedText := sl[i];
    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    SetElementEditValues(r, 'FULL', slv[2]);
    SetElementEditValues(r, 'Model\MODL', slv[3]);
    if slv[4] <> '' then SetElementNativeValues(r, 'Sound\SNAM', StrToInt64('$' + slv[4]));
    if slv[5] <> '' then SetElementNativeValues(r, 'Sound\ANAM', StrToInt64('$' + slv[5]));
    //Exit;
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind DOOR.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  snam, anam: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'DOOR' then
    Exit;

  // export
  snam := IntToHex(GetElementNativeValues(e, 'SNAM'), 8);
  if Copy(snam, 1, 2) <> '01' then snam := '';
  anam := IntToHex(GetElementNativeValues(e, 'ANAM'), 8);
  if Copy(anam, 1, 2) <> '01' then anam := '';
  line := Format('%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    snam,
    anam
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
