// transfer LIGH from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r: IInterface;
  i: integer;
  flags, formid: Cardinal;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin

    slv.DelimitedText := sl[i];
    
    formid := $00072759; // default - LightSpellLightStatic [LIGH:00072759]

    flags := StrToInt('$' + slv[9]);
    flags := flags and $FFFFFFEF; // remove "unknown" flag from oblivion

    // flicker, Torch01 "Torch" [LIGH:0001D4EC]
    if (flags and ($8 + $40)) > 0 then formid := $0001D4EC;
    
    baserecord := RecordByFormID(FileByIndex(0), formid, True);
    if not Assigned(baserecord) then begin
      AddMessage('Can not find base record ' + IntToHex(formid, 8));
      Result := 1;
      Exit;
    end;


    // copy base record as new
    r := wbCopyElementToFile(baserecord, skywind, True, True);
    if not Assigned(r) then begin
      AddMessage('Unable to copy base record as new');
      Exit;
    end;

    SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
    SetElementEditValues(r, 'EDID', slv[1]);
    if slv[2] <> '' then begin
      Add(r, 'FULL', True);
      SetElementEditValues(r, 'FULL', slv[2]);
    end else
      RemoveElement(r, 'FULL');
    if slv[3] <> '' then SetElementEditValues(r, 'Model\MODL', slv[3]) else begin
      RemoveElement(r, 'Model');
      SetElementEditValues(r, 'OBND\X1', '0');
      SetElementEditValues(r, 'OBND\Y1', '0');
      SetElementEditValues(r, 'OBND\Z1', '0');
      SetElementEditValues(r, 'OBND\X2', '0');
      SetElementEditValues(r, 'OBND\Y2', '0');
      SetElementEditValues(r, 'OBND\Z2', '0');
    end;
    SetElementEditValues(r, 'DATA\Time', slv[4]);
    SetElementEditValues(r, 'DATA\Radius', slv[5]);
    SetElementEditValues(r, 'DATA\Color\Red', slv[6]);
    SetElementEditValues(r, 'DATA\Color\Green', slv[7]);
    SetElementEditValues(r, 'DATA\Color\Blue', slv[8]);
    SetElementNativeValues(r, 'DATA\Flags', flags);
    SetElementEditValues(r, 'DATA\Falloff Exponent', slv[10]);
    SetElementEditValues(r, 'DATA\FOV', slv[11]);
    SetElementEditValues(r, 'DATA\Value', slv[12]);
    SetElementEditValues(r, 'DATA\Weight', slv[13]);
    SetElementEditValues(r, 'FNAM', slv[14]);
    if slv[15] <> '' then begin
      Add(r, 'SNAM', True);
      SetElementNativeValues(r, 'SNAM', StrToInt('$' + slv[15]));
    end else
      RemoveElement(r, 'SNAM');
  end;
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind LIGH.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  snam: string;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'LIGH' then
    Exit;
  
  // export
  snam := IntToHex(GetElementNativeValues(e, 'SNAM'), 8);
  if Copy(snam, 1, 2) <> '01' then snam := '';
  line := Format('%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'FULL'),
    GetElementEditValues(e, 'Model\MODL'),
    GetElementEditValues(e, 'DATA\Time'),
    GetElementEditValues(e, 'DATA\Radius'),
    GetElementEditValues(e, 'DATA\Color\Red'),
    GetElementEditValues(e, 'DATA\Color\Green'),
    GetElementEditValues(e, 'DATA\Color\Blue'),
    IntToHex(GetElementNativeValues(e, 'DATA\Flags'), 8),
    GetElementEditValues(e, 'DATA\Falloff Exponent'),
    GetElementEditValues(e, 'DATA\FOV'),
    GetElementEditValues(e, 'DATA\Value'),
    GetElementEditValues(e, 'DATA\Weight'),
    GetElementEditValues(e, 'FNAM'),
    snam
  ]);
  sl.Add(line);
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
end;

end.
