// transfer LVLC from Morroblivion to Skywind

unit UserScript;

var
  fname: string;
  sl, slv: TStringList;
  slObl, slSky: TStringList;

//============================================================================
function Obl2Sky(FormID: Cardinal): Cardinal;
var
  i: integer;
begin
  if not Assigned(slObl) then begin
    //AddMessage('Initializing converter');
    slObl := TStringList.Create;
    slSky := TStringList.Create;
    sl.LoadFromFile(ProgramPath + 'Edit Scripts\OblivionSkyrim.txt');
    for i := 0 to sl.Count - 1 do begin
      slv.DelimitedText := sl[i];
      if slv.Count < 1 then Continue;
      slObl.Add(slv[0]);
      if slv.Count > 2 then
        slSky.Add(slv[2])
      else
        //0Skeleton1Archer "Skeleton" [NPC_:0100080C]
        slSky.Add('0100080C');
    end;
  end;
  
  if (FormID shr 24) > 0 then begin
    Result := FormID;
    Exit;
  end;
  
  i := slObl.IndexOf(IntToHex64(FormID, 8));
  if i <> -1 then
    Result := StrToInt64('$' + slSky[i])
  else
    Result := $0100080C;

end;

//============================================================================
procedure ImportData;
var
  skywind: IInterface;
  baserecord, r, lst, entry: IInterface;
  i: integer;
begin
  skywind := FileByIndex(FileCount - 1);
  if not SameText(GetFileName(skywind), 'skywind.esp') then begin
    AddMessage('Skywind.esp must be loaded last');
    Result := 1;
    Exit;
  end;

  // LCharDraugr05EbonyMelee2HFSublist [LVLN:0003BCBA]
  baserecord := RecordByFormID(FileByIndex(0), $0003BCBA, True);
  if not Assigned(baserecord) then begin
    AddMessage('Can not find base record');
    Result := 1;
    Exit;
  end;
  
  // init converter
  Obl2Sky(0);
  
  sl.LoadFromFile(fname);
  
  for i := 0 to sl.Count - 1 do begin
    slv.DelimitedText := sl[i];
    if slv[0] <> '+' then begin
      // set count for previous record
      if Assigned(lst) then
        SetElementNativeValues(r, 'LLCT', ElementCount(lst));
      // copy base record as new
      r := wbCopyElementToFile(baserecord, skywind, True, True);
      if not Assigned(r) then begin
        AddMessage('Unable to copy base record as new');
        Exit;
      end;
      SetLoadOrderFormID(r, StrToInt('$' + slv[0]));
      SetElementEditValues(r, 'EDID', slv[1]);
      SetElementEditValues(r, 'LVLD', slv[2]);
      SetElementNativeValues(r, 'LVLF', StrToInt(slv[3]));
      lst := ElementByName(r, 'Leveled List Entries');
      // clear list
      while ElementCount(lst) > 0 do RemoveElement(lst, 0);
    end else begin
      // add new entry at the end of list
      entry := ElementAssign(lst, HighInteger, nil, False);
      SetElementEditValues(entry, 'LVLO\Level', slv[1]);
      SetElementNativeValues(entry, 'LVLO\Reference', Obl2Sky(StrToInt64('$' + slv[2])));
      SetElementEditValues(entry, 'LVLO\Count', slv[3]);
    end;
  end;
  
  // set count for last record
  if Assigned(lst) then
    SetElementNativeValues(r, 'LLCT', ElementCount(lst));
end;  

//============================================================================
function Initialize: integer;
begin
  fname := ProgramPath + 'Edit Scripts\Skywind LVLC.txt';
  sl := TStringList.Create;
  slv := TStringList.Create;
  slv.Delimiter := ';';
  slv.StrictDelimiter := true;

  // import
  if wbGameMode = gmTES5 then begin
    ImportData;
  end;
end;

//============================================================================
function Process(e: IInterface): integer;
var
  line: string;
  i: integer;
  lst, item: IInterface;
begin
  if wbGameMode <> gmTES4 then begin
    Result := 1;
    Exit;
  end;

  if Signature(e) <> 'LVLC' then
    Exit;

  // export
  line := Format('%s;%s;%s;%s', [
    IntToHex64(GetLoadOrderFormID(e), 8),
    GetElementEditValues(e, 'EDID'),
    GetElementEditValues(e, 'LVLD'),
    IntToStr(GetElementNativeValues(e, 'LVLF'))
  ]);
  sl.Add(line);
  
  lst := ElementByName(e, 'Leveled List Entries');
  if not Assigned(lst) then
    Exit;
  
  for i := 0 to ElementCount(lst) - 1 do begin
    item  := ElementByIndex(lst, i);
    line := Format('+;%s;%s;%s', [
      GetElementEditValues(item, 'Level'),
      IntToHex64(GetElementNativeValues(item, 'Reference'), 8),
      GetElementEditValues(item, 'Count')
    ]);
    sl.Add(line);
  end;
   
end;

//============================================================================
function Finalize: integer;
begin
  if wbGameMode = gmTES4 then begin
    AddMessage('Saving ' + fname);
    sl.SaveToFile(fname);
  end;
  sl.Free;
  slv.Free;
  if Assigned(slObl) then slObl.Free;
  if Assigned(slSky) then slSky.Free;
end;

end.
